﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Autofac;
using Autofac.Extensions.DependencyInjection;
using DIMultipleImplementationOfServiceAutofacCore21.Controllers;
using DIMultipleImplementationOfServiceAutofacCore21.Services;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;

namespace DIMultipleImplementationOfServiceAutofacCore21
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }
        public ILifetimeScope AutofacContainer { get; private set; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public IServiceProvider ConfigureServices(IServiceCollection services)
        {
            services.AddMvc().SetCompatibilityVersion(CompatibilityVersion.Version_2_1)
                             .AddControllersAsServices();

            services.AddScoped<IProductsService, ProductsService>();

            var builder = new ContainerBuilder();
            builder.Populate(services);

            builder.RegisterType<NegativeValuesService>().As<IValuesService>().AsSelf();
            builder.RegisterType<PositiveValuesService>().As<IValuesService>().AsSelf();
            builder.Register(ctx => new NegativeValuesController(ctx.Resolve<NegativeValuesService>()));
            builder.Register(ctx => new PositiveValuesController(ctx.Resolve<PositiveValuesService>()));

            AutofacContainer = builder.Build();
            
            return new AutofacServiceProvider(AutofacContainer);
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IHostingEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseMvc();
        }
    }
}
