using Autofac;
using Autofac.Core;
using Autofac.Extensions.DependencyInjection;
using DIMultipleImplementationOfServiceAutofacCore3.Controllers;
using DIMultipleImplementationOfServiceAutofacCore3.Services;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;

namespace DIMultipleImplementationOfServiceAutofacCore3
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddScoped<IProductsService, ProductsService>();

            services.AddControllers();
            services.AddMvc().AddControllersAsServices();
        }

        public void ConfigureContainer(ContainerBuilder builder)
        {
            builder.RegisterType<NegativeValuesService>().As<IValuesService>().AsSelf();
            builder.RegisterType<PositiveValuesService>().As<IValuesService>().AsSelf();

            builder.Register(ctx => new NegativeValuesController(ctx.Resolve<NegativeValuesService>()));
            builder.Register(ctx => new PositiveValuesController(ctx.Resolve<PositiveValuesService>()));
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseRouting();

            // app.UseAuthorization();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
        }
    }
}
